@@ -32,14 +32,15 @@
 
 #include <pango/pangocairo.h>
 
+/* Have a space between letters to make sure ligatures aren't used when caching the glyphs: bug 793391. */
 #define VTE_DRAW_SINGLE_WIDE_CHARACTERS	\
-					" !\"#$%&'()*+,-./" \
-					"0123456789" \
-					":;<=>?@" \
-					"ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
-					"[\\]^_`" \
-					"abcdefghijklmnopqrstuvwxyz" \
-					"{|}~" \
+					"  ! \" # $ % & ' ( ) * + , - . / " \
+					"0 1 2 3 4 5 6 7 8 9 " \
+					": ; < = > ? @ " \
+					"A B C D E F G H I J K L M N O P Q R S T U V W X Y Z " \
+					"[ \\ ] ^ _ ` " \
+					"a b c d e f g h i j k l m n o p q r s t u v w x y z " \
+					"{ | } ~ " \
 					""
 
 static inline bool
@@ -147,9 +148,6 @@ _vte_double_equal(double a,
  *
  * Setting this to a large value can cause dramatic slow-downs for some
  * xservers (notably fglrx), see bug #410534.
- *
- * Moreover, setting it larger than %VTE_DRAW_MAX_LENGTH is nonsensical,
- * as the higher layers will not submit runs longer than that value.
  */
 #define MAX_RUN_LENGTH 100
 
@@ -981,12 +979,6 @@ _vte_draw_get_char_edges (struct _vte_draw *draw, vteunistr c, int columns, guin
                 *right = l + w;
 }
 
-gboolean
-_vte_draw_has_bold (struct _vte_draw *draw, guint style)
-{
-	return (draw->fonts[style ^ VTE_DRAW_BOLD] != draw->fonts[style]);
-}
-
 /* Check if a unicode character is actually a graphic character we draw
  * ourselves to handle cases where fonts don't have glyphs for them. */
 static gboolean
@@ -1560,22 +1552,6 @@ _vte_draw_text (struct _vte_draw *draw,
 	}
 
 	_vte_draw_text_internal (draw, requests, n_requests, color, alpha, style);
-
-	/* handle fonts that lack a bold face by double-striking */
-	if ((style & VTE_DRAW_BOLD) && !_vte_draw_has_bold (draw, style)) {
-		gsize i;
-
-		/* Take a step to the right. */
-		for (i = 0; i < n_requests; i++) {
-			requests[i].x++;
-		}
-		_vte_draw_text_internal (draw, requests,
-					   n_requests, color, alpha, style);
-		/* Now take a step back. */
-		for (i = 0; i < n_requests; i++) {
-			requests[i].x--;
-		}
-	}
 }
 
 /* The following two functions are unused since commit 154abade902850afb44115cccf8fcac51fc082f0,
